package de.hdm_stuttgart.de.testing.dom;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import de.hdm_stuttgart.mi.sda1.saxhtml.tools.SaxFilter;
import de.hdm_stuttgart.mi.sda1.saxhtml.tools.StandardErrorHandler;

/**
 * Turning &lt;memo&gt; documents to HTML as being shown in sample document
 * memo.xml.2.sample.html
 */
@SuppressWarnings("javadoc")
public abstract class SaxFilterTest {

	public final String xmlInputFileName, htmlOutputFileName;
	final SaxFilter saxHandler;
	protected static String errorInitString = null;

	protected static Element htmlRootElement = null;

	public SaxFilterTest(final String xmlInputFileName,
						final SaxFilter saxHandler, 
						final String resultFileExtension) {
		this.xmlInputFileName = xmlInputFileName;
		htmlOutputFileName = xmlInputFileName + resultFileExtension;
		this.saxHandler = saxHandler;

		if (null == htmlRootElement) {

			final PrintStream out;

			try {
				out = new PrintStream(htmlOutputFileName);
			} catch (FileNotFoundException e1) {
				errorInitString = "Unable to open file '" + htmlOutputFileName
						+ "' for writing";
				return;
			}
			saxHandler.setOutputStream(out);

			final XMLReader xmlReader;
			try {
				final SAXParserFactory saxPf = SAXParserFactory.newInstance();
				final SAXParser saxParser = saxPf.newSAXParser();
				xmlReader = saxParser.getXMLReader();

				xmlReader.setContentHandler(saxHandler);
				xmlReader.setErrorHandler(new StandardErrorHandler());

			} catch (SAXException | ParserConfigurationException e) {
				errorInitString = "Unable to create XMLparser instance: " + e.getLocalizedMessage();
				return;
			}
			try {
				xmlReader.parse(xmlInputFileName);
			} catch (SAXException | IOException e) {
				errorInitString = "Unable parse file '" + xmlInputFileName
						+ "': " + e.getLocalizedMessage();
				return;
			} 
			out.close();
			printResultToStdout();

			// Now parse the result for further analysis
			//
			final SAXBuilder parser = new SAXBuilder();

			try {
				htmlRootElement = parser.build(htmlOutputFileName)
						.getRootElement();
			} catch (JDOMException | IOException e1) {
				System.err.print(
				errorInitString = "Unable to parse file '" + htmlOutputFileName
						+ "':\n" + e1.getLocalizedMessage() +
						"\n, exiting");
				
				System.exit(1);
			}
		}
	}

	private void printResultToStdout() {
		try {
			BufferedReader saxResult = new BufferedReader(new FileReader(
					htmlOutputFileName));
			String line;
			while (null != (line = saxResult.readLine())) {
				System.out.println(line);
			}
			saxResult.close();
		} catch (IOException e) {
			errorInitString = "Unable to open expected result output file '"
					+ htmlOutputFileName + "' :" + e.getLocalizedMessage();
		}
	}

}